/*
!=======================================================================
!
!  PROGRAM  PHASE-Viewer  (PHASE-Viewer 2014.01 ver.3.3.0)
!
!  Created on 2006/12/12 16:55:11
!  AUTHOR(S): KOGA, Junichiro
!  File : Hexagonal.java
!  
!  Contact address :  Phase System Consortium
!                     E-mail: phase_system@nims.go.jp URL https://azuma.nims.go.jp
!
!
!   Since 2002, this program set had been intensively developed as a part of the following 
!  national projects supported by the Ministry of Education, Culture, Sports, Science and
!  Technology (MEXT) of Japan; "Frontier Simulation Software for Industrial Science
!  (FSIS)" from 2002 to 2005, "Revolutionary Simulation Software (RSS21)" from 2006 to
!  2008. "Research and Development of Innovative Simulation Software (RISS)" from 2008
!  to 2013. These projects is lead by the Center for Research on Innovative Simulation 
!  Software (CISS), the Institute of Industrial Science (IIS), the University of Tokyo.
!   Since 2013, this program set has been further developed centering on PHASE System
!  Consortium. 
!   The activity of development of this program set has been supervised by Takahisa Ohno.
!
!=======================================================================
 */
package ciss.phase_viewer.atomcoord.symmetry;

public class Hexagonal extends BravaisLattice {
    private org.apache.log4j.Logger logger = org.apache.log4j.Logger
            .getLogger(Hexagonal.class.getName());

    public String getName() {
        if (latticeSystem.equals(RHOMBOHEDRAL))
            return "rhombohedral";
        return "hexagonal";
    }

    public double[][] getPrimitiveLatticeVector() {
        if (latticeSystem.equals(RHOMBOHEDRAL))
            return new double[][] {
                    { 0.5 * a, (0.5 / Math.sqrt(3.0)) * a, c / 3.0 },
                    { -0.5 * a, (0.5 / Math.sqrt(3.0)) * a, c / 3.0 },
                    { 0, -a / Math.sqrt(3.0), c / 3.0 } };
        return getGenericPrimitiveVector();
    }

    protected boolean isThisForMe(double a, double b, double c, double alpha,
            double beta, double gamma) {
        boolean len = epsilonEquals(a, b) && !epsilonEquals(a, c);
        boolean ang = epsilonEquals(gamma, 120);
        return len && ang;
    }

    public String[] getLatticeSystemCandidate() {
        return new String[] { HEXAGONAL, RHOMBOHEDRAL };
    }

    protected BravaisLattice tryToCreateBravaisLattice(double[] avec,
            double[] bvec, double[] cvec) {
        if (epsilonEquals(a, b) && !epsilonEquals(a, c)
                && epsilonEquals(gamma, 120) && epsilonEquals(alpha, 90)
                && epsilonEquals(beta, 90)) {
            latticeSystem = HEXAGONAL;
            return this;
        }

        if (epsilonEquals(a, c) && epsilonEquals(b, c)
                && epsilonEquals(alpha, beta) && epsilonEquals(beta, gamma)
                && !epsilonEquals(alpha, 90) && !epsilonEquals(beta, 90)
                && !epsilonEquals(gamma, 90)) {

            Hexagonal hexa = new Hexagonal();
            hexa.a = hexa.b = avec[0] * 2.0;
            hexa.c = avec[2] * 3.0;
            hexa.alpha = 90;
            hexa.beta = 90;
            hexa.gamma = 120;
            hexa.latticeSystem = RHOMBOHEDRAL;
            return hexa;
        }

        return null;
    }

    public String getCrystalSystem() {
        return "hexagonal";
    }

}
